<?php  

defined('C5_EXECUTE') or die(_("Access Denied.")); 

Loader::model('file_attributes'); 
Loader::model('file_list'); 
Loader::model('file_set'); 

class TonyImageSearchBlockController extends BlockController { 

	protected $btTable = 'btTonyImageSearch';
	protected $btInterfaceWidth = "500";
	protected $btInterfaceHeight = "350"; 
	
	
	/** 
	 * Used for localization. If we want to localize the name/description we have to include this
	 */
	public function getBlockTypeDescription() {
		return t("Find images and files by keyword, tag, or attribute");
	}
	
	public function getBlockTypeName() {
		return t("Image/File Search");
	}	

	public function getDefaultSortableOpts(){
		$details=array();
		$details['title'] = t('Title');
		$details['filename'] = t('File Name');
		$details['date'] = t('Date Added');
		return $details;
	}

	public function getDefaultSearchableAttrs(){
		$details=array();
		$details['Keywords']=t('Keywords'); 
		
		//$details['Title']=t('Title');
		//$details['Description']=t('Description');
		$details['Tags']=t('Tags');
		$details['Date']=t('Date Added');
		$details['Sets']=t('Sets');
		//$details['File']=t('File Name');
		//$details['FileSize']=t('File Size');  
		$details['FileType']=t('File Type'); 
		$details['FID']=t('File ID'); 
		return $details;
	}	

	public function getDefaultDetails(){
		$details=array();
		$details['Image']=t('Image');
		$details['Title']=t('Title');
		$details['Description']=t('Description');
		$details['Tags']=t('Tags');
		$details['Date']=t('Date Added');
		$details['File']=t('File Name');
		$details['FileSize']=t('File Size'); 
		$details['Download']=t('Download Link'); 
		return $details;
	}	
	
	public function getDefaultSearchableFileSets(){ 
		$u = new User();
		$db = Loader::db(); 
		$sets = array(); 
		$r = $db->Execute('select * from FileSets where fsType = ? or (fsType in (?, ?) and uID = ?) order by fsName asc', array(FileSet::TYPE_PUBLIC, FileSet::TYPE_STARRED, FileSet::TYPE_PRIVATE, $u->getUserID())); 
		while ($row = $r->FetchRow()) {
			$fs = new FileSet();
			foreach($row as $key => $value) {
				$fs->{$key} = $value;
			}
			$sets[] = $fs; 
		}
		//ksort($sets);
		return $sets; 
	}
	
	//this is used in block edit mode instead of FileSet::getMySets() so that file sets wont be lost if another user edits the block 
	public function getDisplayableFilesSets(){
		$db = Loader::db();
		$fileSetsData = $db->GetAll('select * from FileSets order by fsName asc'); 
		$sets=array();
		foreach ($fileSetsData as $fileSetData) {
			$fs = new FileSet();
			foreach($fileSetData as $key => $value) {
				$fs->{$key} = $value;
			}
			$fsp = new Permissions($fs);
			if ($fsp->canSearchFiles()) {
				$sets[] = $fs;
			}
		}
		//ksort($sets);
		return $sets;
	}	

	
	public function getJavaScriptStrings() {
		return array();
	} 
	
	public function save($args) { 
		$db = Loader::db(); 
		
		$searchableAttrsClean=array();
		$searchableAttrs = $args['searchAttrs'];
		if(is_array($searchableAttrs)) foreach($searchableAttrs as $handle){
			$searchableAttrsClean[]=$handle;
		}
		$args['searchableAttributes'] =  join(',',$searchableAttrsClean);

		$args['setIds']=''; 
		if( is_array($args['fsIDs']) ){
			$cleanSetIds=array();
			foreach($args['fsIDs'] as $fsID) $cleanSetIds[]=intval($fsID);
			$args['setIds']=join(',',$cleanSetIds);
		}
		
		$args['excludeSetIds']=''; 
		if( is_array($args['excluded_fsIDs']) ){
			$cleanSetIds=array();
			foreach($args['excluded_fsIDs'] as $fsID) $cleanSetIds[]=intval($fsID);
			$args['excludeSetIds']=join(',',$cleanSetIds);
		} 
		
		$args['fileTypeIds']='';
		if( is_array($args['typeIDs']) ){
			$cleanFileTypesIds=array();
			foreach($args['typeIDs'] as $typeID) $cleanFileTypesIds[]=intval($typeID);
			$args['fileTypeIds']=join(',',$cleanFileTypesIds);
		}		
		
		$args['showSortBy'] = intval($args['showSortBy']);
		$args['sortByOpts'] = ( is_array($args['sortableBy']) ) ? join(',',$args['sortableBy']) :''; 
		
		
		$shownAttrsClean=array();
		$showAttrs = $args['shownAttrs'];
		if(is_array($showAttrs)) foreach($showAttrs as $handle){
			$shownAttrsClean[]=$handle;
		}
		$args['shownAttributes'] =  join(',',$shownAttrsClean); 
		
		$shownLabelsClean=array();
		$showLabels = $args['shownLabels'];
		if(is_array($showLabels)) foreach($showLabels as $handle){
			$shownLabelsClean[]=$handle;
		}
		$args['shownLabels'] =  join(',',$shownLabelsClean);	
		
		$args['hideEmptyAttrs']=intval($args['hideEmptyAttrs']); 
		
		$args['detailTargetCID'] = (!intval($args['showDetailTarget']) || !intval($args['detailTargetCID']) ) ? 0 : intval($args['detailTargetCID']); 
		
		$args['searchTargetCID'] = (!intval($args['showSearchTarget']) || !intval($args['searchTargetCID']) ) ? 0 : intval($args['searchTargetCID']); 
		
		$args['displayOrderDesc'] = (intval($args['displayOrderDesc']) > 0) ? intval($args['displayOrderDesc']) : 0;	
		
		$args['orderBy'] = (strlen($args['orderBy']) > 0) ? $args['orderBy'] : 'title'; 		
		
		parent::save($args); 
	} 
	
	public function view(){ 		
	
		global $c;	
		$db=Loader::db();  
		$searchedSetIds = array();  
		
		if($_REQUEST['resortQStr']){ 
			$qStrData = unserialize( stripslashes($_REQUEST['resortQStr']) ); 
			foreach($qStrData as $key=>$val){
				if($key=='sortBy') continue; 
				$_GET[$key] = $val;
				$_REQUEST[$key] = $val; 
			}
		}
		
		//searched image sets
		$availableSetIds = ( strlen($this->setIds)) ? explode( ',', $this->setIds ) : array();  
		$filteredSets = (!isset($_REQUEST['sets'])) ? array() : $_REQUEST['sets']; 
		if( !is_array($filteredSets) ) $filteredSets=explode(',',$filteredSets);
		$searchedSetIds = $filteredSets;		
		
		if( $this->mode!='search' ){ 
			
			$fileList = new FileList();  
			$searchAttrs = explode( ',' , $this->searchableAttributes );
			
			if( $this->imagesOnly )
				$fileList->filterByType(FileType::T_IMAGE);
			
			//search
					
			//filter by keywords
			$fID=$_REQUEST['fID'];  
			if( intval($fID)  && in_array('FID',$searchAttrs) ){  
				$fileList->filter( false, 'f.fID='.intval($fID)  ); 
				$filterByFID=1; 
			} 
			
			//filter by keywords
			$keywords=$_REQUEST['keywords'];  
			if( $keywords  && in_array('Keywords',$searchAttrs) ){  
				$fileList->filterByKeywords( $keywords );
				$filterByKeywords=1;
			} 
			
			//filter by tag
			$tag=$_REQUEST['tag']; 
			if( $tag && in_array('Tags',$searchAttrs) ){
				$fileList->filterByTag( $tag );
				$filterByTag=1;
			}
			
			//filter by date added search
			$dateFrom = $_REQUEST['date_from']; 
			if ($dateFrom != '' && in_array('Date',$searchAttrs)  ) {
				$dateFrom = date('Y-m-d', strtotime($dateFrom));
				$fileList->filterByDateAdded($dateFrom, '>=');
				$dateFrom .= ' 00:00:00';
				$filterByDateFrom = 1;
			}
			$dateTo = $_REQUEST['date_to']; 
			if ($dateTo != '' && in_array('Date',$searchAttrs)  ) {
				$dateTo = date('Y-m-d', strtotime($dateTo));
				$dateTo .= ' 23:59:59';			
				$fileList->filterByDateAdded($dateTo, '<=');
				$filterByDateTo = 1;
			} 
			
			//Intersect Sets Approach
			if( count($filteredSets) ){ 
				foreach($filteredSets as $setId){ 
					if( count($availableSetIds) && !in_array( $setId,$availableSetIds) ) continue; 
					$fs = FileSet::getById(intval($setId)); 
					if( !is_object($fs) || !$fs->getFileSetID() ) continue; 
					$fileList->filterBySet($fs); 
					$searchedSetIds[] = intval($setId); 
				} 
			}   
			
			//Filter by Sets 
			//otherwise make sure files are in at least one of the administrator specified sets  
			//Combine Sets Apprach		
			if( !count($seachedSetIds) && count($availableSetIds) ){  
				$fileSetJoinClauses=array(); 
				foreach( $availableSetIds as $setId ){ 
					$fileSet = FileSet::getByID( intval($setId) );
					if(!intval($setId) || !is_object($fileSet) ) continue; 
					$tableAliasName='fsf'.intval($setId);
					$fileList->addToQuery("left join FileSetFiles {$tableAliasName} on {$tableAliasName}.fID = f.fID"); 		
					$fileSetJoinClauses[]="{$tableAliasName}.fsID=".intval($setId);
				}
				if(count($fileSetJoinClauses))
					$fileList->filter(false, '('.join(' OR ',$fileSetJoinClauses).')'); 
			}
			
			//exclude by set
			$excludedSetIdsArray = strlen($this->excludeSetIds) ? explode( ',', $this->excludeSetIds ) : array(); 
			$excludedSetIdsArray = array_unique($excludedSetIdsArray); 
			if( count($excludedSetIdsArray) ){
				$cleanExcludeSetIds=array();
				foreach($excludedSetIdsArray as $sID) if(intval($sID)) $cleanExcludeSetIds[]=intval($sID);
				//$fileList->filter( 'excludeSets.fsID', $cleanExcludeSetIds, '!=');
				if( count($cleanExcludeSetIds) ){
					$fileList->addToQuery( "LEFT JOIN FileSetFiles excludeSets ON excludeSets.fID = f.fID" );  
					$fileList->filter(false, '( excludeSets.fsID IS NULL OR excludeSets.fsID NOT IN ('.join(',',$cleanExcludeSetIds).') )' );
				}
			}			
			
			//filter by custom file attribute
			//make sure that the attribute being searched is allows to be searched
			$searchedAttrIds=array();  
			if( is_array($_REQUEST['akID'])) foreach($_REQUEST['akID'] as $akID=>$val ){
				if( !intval($akID) ) continue; 
				$fak = FileAttributeKey::get( $akID ); 
				if(!is_object($fak)) continue; 
				if( in_array($fak->getAttributeKeyHandle(), $searchAttrs) ) $searchedAttrIds[]=intval($akID);
			}
			$searchedAttrIds=array_unique($searchedAttrIds); 
			foreach($searchedAttrIds as $akID){ 	
				$fak = FileAttributeKey::get( $akID );  
				$type = $fak->getAttributeType(); 
				if(!is_object($type)) continue; 
				
				//all this chunk is related to a bug on the select attribute type when no option is given 
				if( $type->getAttributeTypeHandle() == 'select'){
					$cleanSelectOptIds = array();
					$dirtySelectOptIds = $_REQUEST['akID'][$akID]['atSelectOptionID']; 
					if( is_array($dirtySelectOptIds) ){ 
						foreach($dirtySelectOptIds as $optId){ 
							if( intval($optId) ) 
								$cleanSelectOptIds[]=intval($optId);
						}
					}
					if( !count($cleanSelectOptIds) ) continue;
					$_REQUEST['akID'][$akID]['atSelectOptionID'] = $cleanSelectOptIds; 
				}
				$filteredAttrsData['akID'][$akID] = $_REQUEST['akID'][$akID]; 
 				$cnt = $type->getController();
				$cnt->setAttributeKey( $fak ); 
				$cnt->searchForm($fileList); 
			} 
			
			//File type search   
			$type = intval($_REQUEST['fType']);
			$fileTypeIdsArray = ( !strlen($this->fileTypeIds) )?'':explode( ',', $this->fileTypeIds); 
			if( !is_array($fileTypeIdsArray)) $fileTypeIdsArray=array();
			if( (intval(type) && in_array('FileType',$searchAttrs)) && 
				!count($fileTypeIdsArray) || in_array($type,$fileTypeIdsArray) ){
				$fileList->filterByType($type); 
				$filterByType=1; 
			}elseif( count($fileTypeIdsArray) ){
				//$fileList->filter('fv.fvType', $fileTypeIdsArray, '=');
				$fileList->filterByType( $fileTypeIdsArray); 
			}
				
			
			//sorting 
			$sortBy = $_REQUEST['sortBy']; 
			$sortAscDesc = 'asc';
			$sortableOpts = $this->getDefaultSortableOpts();
			$sortableByArray = explode( ',', $this->sortByOpts );  
			if( !strlen($sortBy) || ( !in_array($sortBy,$sortableByArray))  ){ //!array_key_exists($sortBy,$sortableOpts) &&
				$sortBy=$this->displayOrder;  
				$sortAscDesc = (intval($this->displayOrderDesc)>0) ? 'desc' : 'asc';
			}  
			if($sortBy=='date'){
				$fileList->sortBy('fDateAdded', $sortAscDesc);
			}elseif($sortBy=='filename'){
				$fileList->sortBy('fvFilename', $sortAscDesc);
			}elseif( substr($sortBy,0,4)=='fak_' ){
				$fakID=substr($sortBy,4); 
				$ak =  FileAttributeKey::get( intval($fakID) );  //->getAttributeKeyHandle()
				if(is_object($ak)) $fileList->sortByAttributeKey( $ak, $sortAscDesc);  
			}else{ // "title"
				$fileList->sortBy('fvTitle', $sortAscDesc ); 
			}	
		
			//Paging 
			$pagingVar = 'pg_bID_'.intval($this->bID);
			$defaultPageSize = (!intval($this->max_rows)) ? 50 : intval($this->max_rows)*intval($this->columns); 
			$pageSize= (intval($_REQUEST['pageSize'])>0 ) ? intval($_REQUEST['pageSize']) : $defaultPageSize;			
			$pageNum=(intval($_REQUEST[$pagingVar])>0)?intval($_REQUEST[$pagingVar]):1; 		
			$filesCount = $fileList->getTotal();
			
			$pagination = Loader::helper('pagination');	 
			$pagination->queryStringPagingVariable=$pagingVar;			
			$page_base=$c->getCollectionPath();
			if(!strstr($page_base,'?')) $page_base.='?';
			$paging_qstr = '&sortBy='.$sortBy.'&pageSize='.$pageSize; 
			
			if( $filterByFID ) $paging_qstr.='&fID='.intval($_REQUEST['fID']);  
			if( $filterByType ) $paging_qstr.='&fType='.intval($type); 
			if( $filterByTag ) $paging_qstr.='&tag='.urlencode($tag);
			if( $filterByKeywords ) $paging_qstr.='&keywords='.urlencode($keywords); 
			if( $filterByDateFrom ) $paging_qstr.='&date_from='.urlencode($_REQUEST['date_from']); 
			if( $filterByDateTo ) $paging_qstr.='&date_to='.urlencode($_REQUEST['date_to']); 
			if( count($searchedSetIds) ) $paging_qstr.= '&sets[]='.join('&sets[]=',array_unique($searchedSetIds) ); 
			if( count($filteredAttrsData) ) $paging_qstr .= '&'.http_build_query($filteredAttrsData);  
			
			$paging_url = $page_base.$paging_qstr; 
			
			
			//$paging_url.= '&department='.urlencode($department).'&company='.urlencode($company);
			$pagination->init( $pageNum, $filesCount, View::url($paging_url), $pageSize );			
			$files = $fileList->get( $pageSize, $pagination->result_offset );	
	
			//get attribute keys to display
			$attrKeysIDsArray = explode( ',', $this->shownAttributes );
			$fileCols=array();
			foreach($attrKeysIDsArray as $attrKeysID){
				//if it's an integer, we'll assume it's for the attribute key
				if( intval($attrKeysID)>0 ){
					$fileAttrKey=FileAttributeKey::get($attrKeysID);
					if(!$fileAttrKey) continue;
					$fileCols[]=$fileAttrKey;
				//otherwise, it's one of the default file properties, and we'll treat it as a string
				}elseif(strlen($attrKeysID)){
					$fileCols[]=$attrKeysID;
				}
			}
			$this->set('fileCols',$fileCols);	
			$this->set('filesCount',$filesCount);						
			$this->set('files',$files);
			$this->set('pageSize', $pageSize);
			$this->set('paginator', $pagination);
			$this->set('new_paging_url', str_replace('%pageNum%','1',$paging_url) ); 
			$this->set('paging_url', $paging_url );
			$this->set('page_base', $page_base );
			$this->set('paging_qstr', $paging_qstr ); 
		
		}// mode not search only

		$this->set('sortBy', $sortBy); 
		$this->set('listMode',$this->listMode);	
		$this->set('searchedSetIds',$searchedSetIds);	
	}
	

	function getImageLink($fileVersion,$linkTargetUrl){ 
	
		if( $this->linkMode=='off' ) return '';
		
		if( $this->linkMode=='download' ) return $fileVersion->getDownloadURL();
		
		if( $this->linkMode=='open' || !$this->linkMode ) return $fileVersion->getURL();
	
		$link = $fileVersion->getAttribute($this->linkAttributeHandle); 
		if( intval($link) ){
			$nh = Loader::helper('navigation');  
			$linkPage = Page::getById(intval($link));
			if( is_object($linkPage) ) $link = $nh->getLinkToCollection($linkPage); 
		}
		if(strlen($link)) return $link; 
		if(!$linkTargetUrl) return '';
		return View::url($linkTargetUrl).'fID='.$fileVersion->getFileID();
	}	
	
}

?>
